/*
 * TODO: All functionality in this module should be moved to CPBGridUtil, or into
 * _BaseComponentGrid directly.
 */
define(
      "sas-cpb/datagrid/CPBDataGrid", ['sas-cpb/cpb', 'sas-cpb/datagrid/RefreshingGridTracker', 'sas-cpb/datagrid/MultiColumnSortingQueryEngine',
       'dijit/registry', 'dojo/store/DataStore', 'dojo/store/JsonRest', 'dojo/store/Memory',
       'dojo/store/util/QueryResults', 'dojo/data/ItemFileWriteStore', 'dojo/_base/declare', 'dojo/domReady!'],
      function(cpb, RefreshingGridTracker, MultiColumnSortingQueryEngine,
            registry, DataStore, JsonRest, Memory,
            QueryResults, ItemFileWriteStore, declare) {

var CPBDataGrid = {};

CPBDataGrid.storeMap = {};
CPBDataGrid.gridMap = {};

CPBDataGrid.getGrid = function(gridName)
{
   return registry.byId(gridName);
   //return CPBDataGrid.gridMap[gridName];
};

CPBDataGrid.clearGridAndReload = function(grid)
{
   if (isString(grid))
   {
      grid = CPBDataGrid.getGrid(grid);
   }
   cpb.consoleLog("CPBDataGrid.clearGridAndReload:: grid, " + grid.cpb_id + " visible?? " + grid.visible);
   if (grid.visible == true)
   {
//      grid.refresh();
      // TODO: Remove when dgrid issue #749 is fixed.  See _BaseComponentGrid for more info.
      grid._refreshAndKeepPage();
   }
};

CPBDataGrid.getSelectedRowKeys = function(grid, keyColumnName)
{
   if (isString(grid))
   {
      grid = CPBDataGrid.getGrid(grid);
   }
   return grid.cpb_getSelectedRowValues(keyColumnName);
};

CPBDataGrid.createMemoryStore = function(gridName, data)
{
   var store = new Memory({ data: data });
   CPBDataGrid.storeMap[gridName] = store;
   return CPBDataGrid.getStore(gridName);
};

CPBDataGrid.createStore = function(gridName, url, useUIContext)
{
   var store = null;
   if (useUIContext)
   {
      store = declare([JsonRest])({
         target: url,
         sortParam: 'sortBy',
         query: function(obj, options) {
            // dgrid issue #752.  options.count is a string instead of a number.
            // This is fixed in dgrid 0.3.12, but we're stuck on 0.3.11.
            if (options && options.count && options.count.substring) {
               options.count = parseInt(options.count);
            }
            // return this.inherited(arguments);
            
            var rs = this.inherited(arguments);
            var deferred = rs.then(function(result) {
               // loop over the docs and apply an id
               for(var i = 0; i < result.length; i++) {
                  result[i].id = i;
               }
               return result;
            });

            var qr = QueryResults(deferred);
            qr.total = rs.total;
            return qr;
         }
      });
      // store = Cache(store, Memory());  // this doesn't seen necessary.  dgrid is always fetching the latest remotely.
   }
   else
   {
      var datastore = new ItemFileWriteStore({ url: url, hierarchical: false });
      var MultiColumnSortDataStore = declare([DataStore], {
         queryEngine: MultiColumnSortingQueryEngine
      });
      store = new MultiColumnSortDataStore({ store: datastore });
   }
   //cpb.consoleLog('Grid ' + gridName + ' fetches data restfully? - ' + useUIContext);
   CPBDataGrid.storeMap[gridName] = store;
   return CPBDataGrid.getStore(gridName);
};

CPBDataGrid.getStore = function(gridName)
{
   return CPBDataGrid.storeMap[gridName];
};

CPBDataGrid.showMaskForAllGridByStore = function(storeName)
{
   runForAllGridsInStore(storeName, function(storeGrid){
      CPBDataGrid.showMask(storeGrid);
   });
};

CPBDataGrid.showMask = function(grid)
{
   if (isString(grid))
   {
      grid = CPBDataGrid.getGrid(grid);
   }
   grid.showMask();
};

CPBDataGrid.hideMaskForAllGridByStore = function(storeName)
{
   runForAllGridsInStore(storeName, function(storeGrid){
      CPBDataGrid.hideMask(storeGrid);
   });
};

CPBDataGrid.hideMask = function(grid)
{
   if (isString(grid))
   {
      grid = CPBDataGrid.getGrid(grid);
   }
   grid.hideMask();
};

isString = function(field)
{
   return (typeof(field) === 'string');
};

/**
 * Refresh a server store, then redraw all grids backed by it.
 */
CPBDataGrid.refreshServerStore = function(storeName)
{
   // show masks on each grid.
   runForAllGridsInStore(storeName, function(storeGrid){
      RefreshingGridTracker.addRefreshingGrid(storeGrid);
      CPBDataGrid.showMask(storeGrid);
   });
   CPBService.clearStoreData(storeName, cpb.getFormParams(),
   {
      async: false,
      callback: function(isCleared)
      {
         // refresh all grids associated with this store.
         runForAllGridsInStore(storeName, function(storeGrid) {
            CPBDataGrid.clearGridAndReload(storeGrid);
         });
         
         // hide masks on each grid.
         runForAllGridsInStore(storeName, function(storeGrid){
            CPBDataGrid.hideMask(storeGrid);
         });
      }
   });
};

/**
 * Refresh a single grid.  Does not reload remote store data.
 */
CPBDataGrid.refreshGrid = function(grid)
{
   if (isString(grid))
   {
      grid = CPBDataGrid.getGrid(grid);
   }
   grid.cpb_refresh();
};

/**
 * Refreshes all grids sharing a single store.  Does not reload remote store data.
 */
CPBDataGrid.refreshGridsSharingStore = function(storeName)
{
   runForAllGridsInStore(storeName, function(storeGrid) {
      CPBDataGrid.showMask(storeGrid);
   });
   runForAllGridsInStore(storeName, function(storeGrid) {
      CPBDataGrid.refreshGrid(storeGrid);
   });
   runForAllGridsInStore(storeName, function(storeGrid) {
      CPBDataGrid.hideMask(storeGrid);
   });
};

CPBDataGrid.getRowCount = function(grid)
{
   if (isString(grid))
   {
      grid = CPBDataGrid.getGrid(grid);
   }
   return grid.cpb_totalRowCount;
};

// private functions

runForAllGridsInStore = function(storeName, process)
{
   for (var gridName in CPBDataGrid.gridMap)
   {
      var storeGrid = CPBDataGrid.getGrid(gridName);
      if (storeGrid.cpb_storeName === storeName)
      {
         process(storeGrid);
      }
   }
};


   return CPBDataGrid;
   
});
