define(
      "sas-cpb/datagrid/RefreshingGridTracker", ['dojo/_base/array', 'dojo/aspect'],
      function(array, aspect) {

// summary:
//   Used when one grid is waiting on other grids to completely refresh themselves.  The first grid will be masked
//   until all "refreshing" grids have completed refreshing, at which point the grid's mask will be removed.
//   This is useful, for example, when one grid refreshes other grids via on-select events.
var RefreshingGridTracker = {};

var refreshingGrids = [],
    gridsWaitingForRefreshes = [];
   
RefreshingGridTracker.addGridWaitingForRefreshes = function(grid) {
   if (array.indexOf(gridsWaitingForRefreshes, grid)==-1) {
console.log('Adding grid waiting for refreshes: ' + grid.cpb_name);
      grid.showMask();
      gridsWaitingForRefreshes.push(grid);
   }
};

RefreshingGridTracker.addRefreshingGrid = function(grid) {
console.log('Adding refreshing grid: ' + grid.cpb_name);
   refreshingGrids.push(grid);
   aspect.after(grid, 'renderArray', function(results, beforeNode) {
      if (typeof results.total === 'number') { // Memory stores
         RefreshingGridTracker.removeRefreshingGrid(grid);
      }
      else { // line count arrives as a Promise - more typical scenario
         results.total.then(function(results) {
            RefreshingGridTracker.removeRefreshingGrid(grid);
         });
      }
   }, true);
};

RefreshingGridTracker.removeRefreshingGrid = function(grid) {
   var index = refreshingGrids.indexOf(grid);
   if (index>-1) {
console.log('Grid done refreshing, removing: ' + grid.cpb_name);
      refreshingGrids.splice(index, 1);
      RefreshingGridTracker.pruneGridsWaitingForRefreshes();
   }
};

RefreshingGridTracker.pruneGridsWaitingForRefreshes = function() {
   if (refreshingGrids.length===0) {
if (gridsWaitingForRefreshes.length>0) {
   console.log('All grids done refreshing! Hiding masks for all these grids:');
}
      array.forEach(gridsWaitingForRefreshes, function(grid) {
console.log('... ' + grid.cpb_name);
         grid.hideMask();
      });
      gridsWaitingForRefreshes = [];
if (gridsWaitingForRefreshes.length>0) {
   console.log('Done');
}
   }
};
   
return RefreshingGridTracker;

});
