/**
 * Utility methods for menu bars and context menus.
 */
define("sas-cpb/menu/CPBMenu", ['sas-cpb/cpb',
   'dojo/_base/array', 'dojo/dom-style', 'dijit/registry',
   'dijit/Toolbar', 'dijit/form/Button', 'dijit/form/DropDownButton',
   'dijit/MenuItem', 'dijit/DropDownMenu', 'dijit/PopupMenuItem', 'dijit/MenuSeparator'],
function(cpb, array, domStyle, registry,
   Toolbar, Button, DropDownButton,
   MenuItem, DropDownMenu, PopupMenuItem, MenuSeparator) {

   // summary:
   //   Adds an icon to a dijit/MenuItem or dijit/DropDownButton when no style for that icon is
   //   defined in the current theme.  This is somewhat of a hack.
   var _addIcon = function(dijit, url, width, height) {
      domStyle.set(dijit.iconNode, 'background', 'no-repeat center url('+url+')');
      domStyle.set(dijit.iconNode, 'display', 'inline-block');
      if (width && height) {
         domStyle.set(dijit.iconNode, {
            width: width + 'px',
            height: height + 'px'
         });
      }
   };

   var CPBMenu = {};
   
   // summary:
   //   Builds and displays a menu bar for the current page.
   CPBMenu.showMenu = function(menuModel) {
      
      //cpb.consoleLog("model == " + JSON.stringify(menuModel));
      var id = menuModel.id || null;
      if (id != null) {
         cpb.consoleLog("Showing menu with id == " + id);
      }
   
      var menuBar = new Toolbar({}, id);
      array.forEach(menuModel.menuItemList, function(modelItem) {

         var child;
         if (modelItem.separator) {
            child = new MenuSeparator({});
         }
         
         else {
            var args = {
               id: modelItem.id,
               disabled: modelItem.disabled || false,
               showLabel: (modelItem.label!=null)//(modelItem.iconClass == null)
            };
            if (modelItem.iconClass) { args.iconClass = modelItem.iconClass; }
            if (modelItem.title) { args.title = modelItem.title; }
            if (modelItem.label) { args.label = modelItem.label; }
            child = modelItem.submenu ? new DropDownButton(args) : new Button(args);
            if (modelItem.submenu) {
               // TODO: What auto-open behavior do we want?
               child.on('click', function(e) {
                  child.openDropDown();
                  child.focus();
               });
               child.dropDown = CPBMenu.buildMenu(child, modelItem.submenu.menuItemList);
            }
            else if (modelItem.onClick) {
               if (typeof(modelItem.onClick) === 'function') {
                  child.onClick = modelItem.onClick;
               }
               else {
                  child.onClick = function() {
                     eval(modelItem.onClick);
                  };
               }
            }
            // Icon was specified by imageSrc (and possible width and height) instead of class
            if (modelItem.imageSrc) {
               _addIcon(child, modelItem.imageSrc, modelItem.imageWidth, modelItem.imageHeight);
            }
         }
         
         menuBar.addChild(child);
         
      });

      menuBar.startup();

      // TODO: Why do we need a manual resize?
      var container = registry.byId('mainBorderContainer');
      if (container) {
         container.resize();
      }
      
   };
   
   // summary:
   //   Recursively builds and returns a DropDownMenu from JSON.
   CPBMenu.buildMenu = function(parent, itemList) {
      
      var menu = new DropDownMenu({style: 'display: none', autoFocus: true });
      //menu.on('mouseLeave', function(e) {
      //   parent.closeDropDown();
      //});
      
      array.forEach(itemList, function(modelItem) {
         
         var child = null;
         
         if (modelItem.separator) {
            child = new MenuSeparator({});
         }
         else {
            
            var args = {
               id: modelItem.id,
               disabled: modelItem.disabled || false,
               showLabel: true
            };
            if (modelItem.iconClass) { args.iconClass = modelItem.iconClass; };
            if (modelItem.title) { args.title = modelItem.title; }
            if (modelItem.label) { args.label = modelItem.label; }
   
            if (modelItem.submenu) {
               args.popup = CPBMenu.buildMenu(parent, modelItem.submenu.menuItemList);
               child = new PopupMenuItem(args);
            }
            else {
               if (modelItem.target) {
                  // We use onClick for the URL or JS, depending on whether target was specified.  Yuck.
                  var target = modelItem.target;
                  var url = modelItem.onClick;
                  if (!url) {
                     cpb.consoleLog('Bad menu item; target specified but no URL: ' + JSON.stringify(modelItem), 'error');
                  }
                  args.onClick = function() {
                     if (target==='_self' || target==='_parent' || target==='_top') {
                        if (cpb.redirectHandler) {
                           cpb.redirectHandler(url);
                           return;
                        }
                     }
                     window.open(url, target);
                  };
               }
               else if (modelItem.onClick) {
                  args.onClick = function() { eval(modelItem.onClick); };
               }
               child = new MenuItem(args);
            }
            
            // Icon was specified by imageSrc (and possible width and height) instead of class
            if (modelItem.imageSrc) {
               _addIcon(child, modelItem.imageSrc, modelItem.imageWidth, modelItem.imageHeight);
            }
            
         }
         
         menu.addChild(child);

      });
      
      return menu;
      
   };
   
   return CPBMenu;
      
});
