var Reports = {};

/** the name of the form (set when the BODY loads) */
Reports.FORM_NAME = null;
Reports.OBJECT_TYPE = null;
Reports.KEY = null;
Reports.ID = null;
Reports.EDITABLE = false;
Reports.REFERRER = null;
Reports.TITLE = null;
Reports.FORM_CONFIG_RK = 0;

/** This is a hack to get view history to work correctly.  It assumes the ENUM name (RR in this case) is the javascript name **/
var Rr = {};
Rr.ID = Reports.ID;

Reports.viewReport = function(/* long */key, /* string */parentKey, /* string */parentType) 
{
   var windowLayout = BrowserUtil.getRequestParameter("windowLayout", "MAIN_DIALOG");
   var url = Global.CONTEXT_PATH + "/Report.do?command=view&viewMode=READ_ONLY&key=" + key
         + "&windowLayout=" + windowLayout;

   if (parentType == "Case")
   {
	   url += "&caseKey=" + parentKey;
	   Case.redirect(url);
   }
   else if (parentType == "Incident")
   {
	   url += "&incidentKey=" + parentKey;
	   Incident.redirect(url);
   }
};

Reports.editReport = function(/* long */key, /* string */parentKey, /* string */parentType) 
{
   var windowLayout = BrowserUtil.getRequestParameter("windowLayout", "MAIN_DIALOG");
   var url = Global.CONTEXT_PATH + "/Report.do?command=view&viewMode=EDITABLE&key=" + key
         + "&windowLayout=" + windowLayout;

   if (parentType == "Case")
   {
	   url += "&caseKey=" + parentKey;
	   Case.redirect(url);
   }
   else if (parentType == "Incident")
   {
	   url += "&incidentKey=" + parentKey;
	   Incident.redirect(url);
   }
};

Reports.addReportFromCase = function(uiContextID, key)
{
   Reports.KEY = key;
   var args = {uiContextID: uiContextID};
  
      SaveChangesConfirmation.confirmSaveChanges(Reports.addReportFromCaseSaveChangesConfirmationCallback, args); 
};

Reports.addReportFromCaseSaveChangesConfirmationCallback = function(saveChangesConfirmation)
{
   var resultCode = saveChangesConfirmation.resultCode;
   
   if (resultCode == SaveChangesConfirmation.ResultCode.YES)
   {
      /* set parameters that will cause new case dialog to appear after save */
     
      var form = Case.getForm();
      FormUtil.updateForm(form, {onload: "newReport"});
         
      // save the case
      App.protectPage();
      Case.save(form);  
        
   }
   else if (resultCode == SaveChangesConfirmation.ResultCode.NOT_DIRTY)
   {
      Reports.newReport();
   }
};

Reports.addReportFromIncident = function(uiContextID, key)
{
   Reports.KEY = key;
   var args = {uiContextID: uiContextID};
  
      SaveChangesConfirmation.confirmSaveChanges(Reports.addReportFromIncidentSaveChangesConfirmationCallback, args); 
};

Reports.addReportFromIncidentSaveChangesConfirmationCallback = function(saveChangesConfirmation)
{
   var resultCode = saveChangesConfirmation.resultCode;
   
   if (resultCode == SaveChangesConfirmation.ResultCode.YES)
   {
      var form = Incident.getForm();
      FormUtil.updateForm(form, {onload: "newReport"});
         
      // save the case
      App.protectPage();
      Incident.save(form);           
   }
   else if (resultCode == SaveChangesConfirmation.ResultCode.NOT_DIRTY)
   {
      Reports.newReport();
   }
};


Reports.newReport = function()
{
   require(['sas-cpb/dialog/Modal'], function(Modal) {
      var url = Global.CONTEXT_PATH + "/Report.do?command=newEntity";
      Modal.show({
         url: url,
         width: 550, height: 250,
         title: AppProperties["newReport.title.txt"]});
   });
};

Reports.getForm = function() 
{
   return document.getElementById(Reports.FORM_NAME);
};

Reports.submitForm = function(valueMap)
{
   var form = Reports.getForm();
   App.handleFileUploaderForSubmit(form);
   FormUtil.submitForm(form, valueMap);
};

Reports.save = function(form)
{
   if (!form)
   {
      form = Reports.getForm();
   }
   
   Reports.handleDefaultSubmit(form);
   form.submit();
};

Reports.saveDraft = function(form)
{
   if (!form)
   {
      form = Reports.getForm();
   }
   
   Reports.handleDraftSubmit(form);
   App.handleFileUploaderForSubmit(form);
   form.submit();
};

Reports.saveNew = function()
{
   var form = Reports.getForm();
   Reports.handleNewDefaultSubmit(form);
   App.handleFileUploaderForSubmit(form);
   form.submit();
};

Reports.handleDefaultSubmit = function(form)
{
   App.protectPage();
   App.handleFileUploaderForSubmit(form);
   FormUtil.updateForm(form, {command: "save"});
};

Reports.handleDraftSubmit = function(form)
{
   FormUtil.updateForm(form, {command: "saveDraft"});
};

Reports.handleNewDefaultSubmit = function(form)
{
   form.target = "_top";
   var formValueMap = {command: "save", isNewEntity: "true"};
   FormUtil.updateForm(form, formValueMap);
};

Reports.comments = function() 
{
   ReportCommon.comments(Reports.OBJECT_TYPE, Reports.KEY, Reports.ID);
};

Reports.attachments = function() 
{
   ReportCommon.attachments(Reports.OBJECT_TYPE, Reports.KEY, Reports.ID);
};

Reports.handleReassignDefaultSubmit = function(form)
{
   FormUtil.updateForm(form, {command :"reassign"});
};

Reports.reassignSubmit = function(form)
{
   var form = Reports.getForm();
   Reports.handleReassignDefaultSubmit(form);
   App.handleFileUploaderForSubmit(form);
   form.submit();
};

Reports.reassignCancel = function()
{
   // Don't load the module twice, reuse the root level one
   window.top.require(['sas-cpb/dialog/Modal'], function(Modal) {
      Modal.hideActiveModal(false);
   });
};

Reports.closeReassignDialog = function()
{
   if (window.parent.Reports !== undefined)
   {
      window.parent.Reports.submitForm();
   }
   else if (window.parent.Search)
   {
      window.parent.Search.refresh();
   }
};

/**
 * This function is called when the page loads.
 */
Reports.handleOnLoad = function()
{
   require(['sas-ecm/shared/Breadcrumb'], function(Breadcrumb) {
      var onload = BrowserUtil.getRequestParameter("onload");
      Breadcrumb.add(Reports.REFERRER, Reports.TITLE);
   
      if (!onload || App.HAS_ERRORS)
      {
         return;
      }
   });
};

/**
 * This method is invoked to open a different URL. If changes have been made to the current page then a "Save Changes"
 * confirmation dialog is shown before redirecting.
 */
Reports.redirect = function(redirectURL, extraCallback)
{
   var callbacks = [ Reports.saveChangesConfirmationCallback ];
   if (extraCallback) {
      // unshift makes it first. this is important because the saveConfirmation will change the window.location.
      callbacks.unshift(extraCallback);
   }
   var args = {redirectURL: redirectURL};
   SaveChangesConfirmation.confirmSaveChanges(callbacks, args);
};

Reports.logoff = function()
{
   Reports.redirect("Logoff");
};

Reports.saveChangesConfirmationCallback = function(saveChangesConfirmation)
{
   App.protectPage();

   var resultCode = saveChangesConfirmation.resultCode;
   
   if (resultCode == SaveChangesConfirmation.ResultCode.YES)
   {
      var separator = "?";
      if (saveChangesConfirmation.args.redirectURL.indexOf("?") > -1)
      {
         separator = "&";
      }
      var redirectURL = saveChangesConfirmation.args.redirectURL + separator + "reportSaved=" + Reports.KEY;
      
      /* save the case but provide a redirect URL */
      var form = Reports.getForm();
      FormUtil.updateForm(form, {redirectURL: redirectURL});
      Reports.saveDraft(form);
   }
   else if ((resultCode == SaveChangesConfirmation.ResultCode.NO) ||
            (resultCode == SaveChangesConfirmation.ResultCode.NOT_DIRTY))
   {
      /* user does not want to save or the page is not dirty */
      window.location = saveChangesConfirmation.args.redirectURL;
   }
};

Reports.correct = function(/* long */key, /* long */selectedKey, /* string */viewMode, /* string */windowLayout)
{
   var args = {key: key, selectedKey: selectedKey, viewMode: viewMode, windowLayout: windowLayout};
   ConfirmReportCorrection.confirmCorrectReport(
		   Reports.confirmCorrectReportCallback, args); 
};

Reports.confirmCorrectReportCallback = function(correctReportConfirmation)
{
   App.protectPage();

   var resultCode = correctReportConfirmation.resultCode;
   
   if (resultCode == ConfirmReportCorrection.ResultCode.YES) {
      var url = "Report.do?command=correct" +
            "&viewMode=" + ConfirmReportCorrection.args.viewMode +
            "&key=" + ConfirmReportCorrection.args.key +
            "&selectedKey=" + ConfirmReportCorrection.args.selectedKey +
            "&windowLayout=" + ConfirmReportCorrection.args.windowLayout;
      Reports.redirect(url);
   }
   else {
	   App.unprotectPage();
   }
   
};

Reports.showParentEntity = function(/* string */action, /* long */key, /* string */windowLayout)
{
	var url = action + "?command=view&viewMode=EDITABLE" + "&key=" + key + "&windowLayout=" + windowLayout; 
   ecmApi.redirect(url);
};
